package com.me.brickbuster.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.EntityType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;

public class Ball extends Entity implements PhysicsBody, CollisionListener {

	public static final float RADIUS = 1.2f;
	public static final Color BALL_COLOR = Color.CHARTREUSE;
	public static final float DEFAULT_SPEED = 45;
	public static final float BOOST_SPEED = 55;
	public static final int BLOCKS_FOR_BOOST = 39;

	public static final float MINIMUM_ANGLE = MathUtils.PI/16;
	public static final float CORRECTION_IMPULSE = 3f;

	private float speed;
	private boolean isStuck = true;
	private boolean touchedPaddle = false;

	private TextureRegion texture;

	private Body body;

	public Ball(PlayState state) {
		super(state, state.paddle.getX(), state.paddle.getY() + Paddle.PADDLE_HEIGHT + RADIUS);
		this.speed = state.bricks.size > BLOCKS_FOR_BOOST? DEFAULT_SPEED : BOOST_SPEED;

		texture = state.textures.findRegion("ball");

		createBody();
	}

	@Override
	public void render(SpriteBatch sb, ShapeRenderer sr) {
		sb.setColor(Color.WHITE);
		sb.draw(texture, pos.x - RADIUS,
				pos.y - RADIUS,
				RADIUS*2,
				RADIUS*2);
	}

	@Override
	public void update(float dt) {
		if (isStuck || deleted) {
			if (!deleted && Gdx.input.justTouched()) {
				launch();
			} else {
				return;
			}
		}
		if (touchedPaddle) {
			paddleCollision();
			touchedPaddle = false;
		}

		if (pos.y + RADIUS < 0) {
			deleted = true;
			return;
		}

		Vector2 velocity = body.getLinearVelocity();
		body.setLinearVelocity(velocity.nor().scl(speed));

		float rad = velocity.angleRad();
		if (Math.abs(rad) < MINIMUM_ANGLE || Math.abs(rad) > MathUtils.PI - MINIMUM_ANGLE) {
			Vector2 impulse = new Vector2(0, rad > 0? CORRECTION_IMPULSE : -CORRECTION_IMPULSE);
			body.applyLinearImpulse(impulse, pos, true);
		}
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void createBody() {
		BodyDef ballBody = new BodyDef();
		ballBody.type = BodyDef.BodyType.DynamicBody;
		ballBody.position.set(pos);

		CircleShape ballShape = new CircleShape();
		ballShape.setRadius(RADIUS);

		FixtureDef ballFixture = new FixtureDef();
		ballFixture.shape = ballShape;
		ballFixture.restitution = 1f;
		ballFixture.friction = 0f;
		ballFixture.density = 1f;

		ballFixture.filter.categoryBits = EntityType.BALL;
		ballFixture.filter.maskBits = EntityType.BOUNDARY | EntityType.BRICK
						| EntityType.PADDLE | EntityType.SHIELD;

		body = state.world.createBody(ballBody);
		body.createFixture(ballFixture);
		body.setUserData(this);

		ballShape.dispose();
	}

	@Override
	public void beginContact(Entity contacted) {
		if (contacted instanceof Shield) {
			contacted.delete();
		}

		if (contacted instanceof Paddle && !isStuck) {
			touchedPaddle = true;
		}
	}

	@Override
	public void endContact(Entity contacted) {
	}

	public Vector2 paddleReflectAngle() {
		Paddle paddle = ((PlayState) state).paddle;
		float rel = (pos.x - paddle.getX()) + (paddle.getWidth()/2);
		float newAngle = MathUtils.PI - (MathUtils.PI * (rel / paddle.getWidth()));
		newAngle = MathUtils.clamp(newAngle, MINIMUM_ANGLE, MathUtils.PI-MINIMUM_ANGLE);
		return new Vector2(MathUtils.cos(newAngle), MathUtils.sin(newAngle));
	}

	public void launch() {
		Vector2 direction;
		if (((PlayState) state).paddle.isSticky()) {
			direction = paddleReflectAngle();
		} else {
			// launch at random angle between 135 and 45 degrees
			float angle = MathUtils.random(MathUtils.PI/2) + MathUtils.PI/4;
			direction = new Vector2(MathUtils.cos(angle), MathUtils.sin(angle));
		}
		body.setLinearVelocity(direction.scl(speed));
		isStuck = false;
	}

	public void paddleCollision() {
		if (((PlayState) state).paddle.isSticky()) {
			isStuck = true;
			body.setLinearVelocity(new Vector2());
			setY(((PlayState) state).paddle.getY() + Paddle.PADDLE_HEIGHT + RADIUS);
			return;
		}
		body.setLinearVelocity(paddleReflectAngle().scl(speed));
	}

	@Override
	public void setX(float x) {
		super.setX(x);
		body.setTransform(pos, 0);
	}

	@Override
	public void setY(float y) {
		super.setY(y);
		body.setTransform(pos, 0);
	}

	public void setSpeed(float speed) {
		this.speed = speed;
	}

	public boolean isStuck() {
		return isStuck;
	}

}
