package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.me.brickbuster.entity.powerup.PowerUpType;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.EntityType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.FieldState;
import com.me.brickbuster.state.PlayState;

public class Brick extends Entity implements PhysicsBody, CollisionListener {

	public static final float BRICK_WIDTH = 5.2f;
	public static final float BRICK_HEIGHT = 2.6f;

	private BrickType type;
	private BrickShape.Shape shape;
	private PowerUpType powerUpType;

	private TextureRegion region;

	private Body body;
	private boolean hitByBall = false;

	public Brick(FieldState state, BrickType type, BrickShape.Shape shape, PowerUpType powerUpType, float x, float y) {
		super(state, x, y);
		this.type = type;
		this.shape = shape;
		this.powerUpType = powerUpType;
		this.region = state.textures.findRegion("brick_" + shape.name());
		createBody();
	}

	@Override
	public void render(SpriteBatch sb, ShapeRenderer sr) {
		sb.setColor(type.getColor());
		sb.draw(region, pos.x - BRICK_WIDTH/2, pos.y - BRICK_HEIGHT/2,
				BRICK_WIDTH/2, BRICK_HEIGHT/2,
				BRICK_WIDTH, BRICK_HEIGHT,
				1f, 1f,
				MathUtils.radiansToDegrees * body.getAngle());
	}

	@Override
	public void update(float dt) {
		if (hitByBall) {
			hit();
			hitByBall = false;
		}

		if (getY() + BRICK_HEIGHT < 0) {
			delete();
		}
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void createBody() {
		BodyDef brickBody = new BodyDef();
		brickBody.type = BodyDef.BodyType.StaticBody;
		brickBody.position.set(pos.cpy());

		int shapeMask = shape.getMask();
		Vector2[] vertices = new Vector2[Integer.bitCount(shapeMask)];
		int v = 0;

		if ((shapeMask & BrickShape.BOTTOM_LEFT) != 0) {
			vertices[v++] = new Vector2(-BRICK_WIDTH/2, -BRICK_HEIGHT/2);
		}
		if ((shapeMask & BrickShape.MIDDLE_LEFT) != 0) {
			vertices[v++] = new Vector2(-BRICK_WIDTH/2, 0);
		}
		if ((shapeMask & BrickShape.TOP_LEFT) != 0) {
			vertices[v++] = new Vector2(-BRICK_WIDTH/2, BRICK_HEIGHT/2);
		}
		if ((shapeMask & BrickShape.TOP_MIDDLE) != 0) {
			vertices[v++] = new Vector2(0, BRICK_HEIGHT/2);
		}
		if ((shapeMask & BrickShape.CENTER) != 0) {
			vertices[v++] = new Vector2(0, 0);
		}
		if ((shapeMask & BrickShape.TOP_RIGHT) != 0) {
			vertices[v++] = new Vector2(BRICK_WIDTH/2, BRICK_HEIGHT/2);
		}
		if ((shapeMask & BrickShape.MIDDLE_RIGHT) != 0) {
			vertices[v++] = new Vector2(BRICK_WIDTH/2, 0);
		}
		if ((shapeMask & BrickShape.BOTTOM_RIGHT) != 0) {
			vertices[v++] = new Vector2(BRICK_WIDTH/2, -BRICK_HEIGHT/2);
		}
		if ((shapeMask & BrickShape.BOTTOM_MIDDLE) != 0) {
			vertices[v++] = new Vector2(0, -BRICK_HEIGHT/2);
		}

		PolygonShape brickShape = new PolygonShape();
		brickShape.set(vertices);

		FixtureDef brickFixture = new FixtureDef();
		brickFixture.shape = brickShape;
		brickFixture.friction = 0f;
		brickFixture.density = 0.5f;

		brickFixture.filter.categoryBits = EntityType.BRICK;
		brickFixture.filter.maskBits = EntityType.BALL | EntityType.BRICK | EntityType.BOUNDARY;


		body = state.world.createBody(brickBody);
		body.createFixture(brickFixture);
		body.setUserData(this);

		brickShape.dispose();
	}

	@Override
	public void beginContact(Entity contacted) {
		if (contacted instanceof Ball) {
			hitByBall = true;
		}
	}

	@Override
	public void endContact(Entity contacted) {
	}

	@Override
	public void setX(float x) {
		super.setX(x);
		body.setTransform(pos, 0);
	}

	@Override
	public void setY(float y) {
		super.setY(y);
		body.setTransform(pos, 0);
	}

	public BrickType getType() {
		return type;
	}

	public BrickShape.Shape getShape() {
		return shape;
	}

	public PowerUpType getPowerUpType() {
		return powerUpType;
	}

	public boolean hit() {
		if (((PlayState) state).bricks.size-1 <= Ball.BLOCKS_FOR_BOOST) {
			for (Ball ball : ((PlayState) state).balls) {
				ball.setSpeed(Ball.BOOST_SPEED);
			}
		}

		float powerUpChance = ((PlayState) state).currentLevel.getRandomPowerUpChance();
		if (powerUpType == null && powerUpChance > 0f && MathUtils.randomBoolean(powerUpChance)) {
			powerUpType = PowerUpType.getWeightedRandom();
		}

		if (powerUpType != null) {
			((PlayState) state).powerUps.add(powerUpType.createInstance((PlayState) state, this));
		}

		deleted = true;
		return true;
	}

}
